// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Arrival;
import com.sncf.checker.bo.Departure;
import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.JointArrDep;
import com.sncf.checker.bo.JointArrDepEntry;
import com.sncf.checker.bo.Parameters;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.ArrivalDAO;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.DepartureDAO;
import com.sncf.checker.io.TrainDAO;

public class DepartureOrderConstraint implements Constraint {

    private final DAOFactory factory;

    public DepartureOrderConstraint(DAOFactory factory) {
        this.factory = factory;
    }

    public boolean validate() throws ROADEFException {
        boolean valid = true;
        HashMap<String, Departure> assigned = new HashMap<String, Departure>();
        HashMap<String, Train> covered =
                new HashMap<String, Train>();
        TrainDAO trainDAO = factory.getTrainDAO();
        ArrivalDAO arrivalDAO = factory.getArrivalDAO();
        DepartureDAO departureDAO = factory.getDepartureDAO();
        Logger logger = Logger.getLogger("logger");
        Parameters parameters = factory.getParametersDAO().getParameters();

        List<Train> trainList = trainDAO.loadAll();
        for (Train train : trainList) {
            for (Fill fill : train.getFill()) {
                if (fill.getEvent().equals(Event.Departure)) {
                    Departure dep = departureDAO.loadByName(fill.getComplement());
                    if (dep != null) {
                        assigned.put(train.getName(), dep);
                        covered.put(dep.getName(), train);
                     }
                }
            }
        }

        for (Train train : trainList) {
            ArrayList<String> assembledTrains = new ArrayList<String>();
            Datetime lastAssemblyTime = null;
            Iterator<Fill> itr = train.getFill().iterator();
            Arrival arrival = train.getArrival();
            if (arrival != null) {
                JointArrDep joint = arrival.getJointArrDep();
                if (joint != null) {
                    for (JointArrDepEntry entry : joint.getEntryList()) {
                        Arrival jointArrival =
                                arrivalDAO.loadByName(entry.getArrDep());
                        assembledTrains.add(jointArrival.getTrain().getName());
                    }
                    while (itr.hasNext()) {
                        Fill fill = itr.next();
                        if (fill.getEvent().equals(Event.Arrival)) {
                            break;
                        }
                    }
                } else {
                    assembledTrains.add(train.getName());
                }
            } else {
                assembledTrains.add(train.getName());
            }
            Fill previous = null;
            while (itr.hasNext()) {
                Fill fill = itr.next();
                if (previous != null) {
                    if (previous.getEvent().equals(Event.ExitResource)
                            && fill.getEvent().equals(Event.EnterResource)
                            && previous.getGate().length() >= 1
                            && fill.getGate().length() >= 1
                            && assembledTrains.size() > 1) {
                        if (previous.getGate().substring(0, 1)
                                .equals(fill.getGate().substring(0, 1))) {
                            ArrayList<String> tmp = new ArrayList<String>();
                            for (String t : assembledTrains) {
                                tmp.add(0, t);
                            }
                            assembledTrains = tmp;
                        }
                    }
                }
                if (fill.getEvent().equals(Event.BegJunction)
                        || fill.getEvent().equals(Event.BegDisjunction)) {
                        // check BeginJunction and BegDisjunction complement
                        ArrayList<String> compareList = new ArrayList<String>(
                                Arrays.asList(fill.getComplement().split("\\+")));
                        boolean correct = true;
                        if (compareList.size() != assembledTrains.size()) {
                            correct = false;
                        } else {
                            for (int i = 0; i < assembledTrains.size(); i++) {
                                if (!assembledTrains.get(i).equals(compareList.get(i))) {
                                    correct = false;
                                }
                            }
                        }
                        if (!correct) {
                            StringBuilder sb = new StringBuilder();
                            for (int i = 0; i < assembledTrains.size(); i++) {
                                sb.append(assembledTrains.get(i));
                                if (i != assembledTrains.size() - 1) {
                                    sb.append("+");
                                }
                            }
                            logger.log(Level.INFO, "Departure order error : "
                                    + "wrong assembled trains list for "
                                    + train.getName() + " at "
                                    + fill.getDate().toString() + " : "
                                    + fill.getComplement() + " ("
                                    + sb.toString() + " expected).");
                            valid = false;
                            break;
                        }
                }
                if (fill.getEvent().equals(Event.EndJunction)
                    || fill.getEvent().equals(Event.EndDisjunction)) {
                    assembledTrains = new ArrayList<String>(
                            Arrays.asList(fill.getComplement().split("\\+")));
                    lastAssemblyTime = fill.getDate();
                }
                if (fill.getEvent().equals(Event.Departure)) {

                    Departure departure = departureDAO.loadByName(
                            fill.getComplement());
                    if (departure != null) {
                        Iterator<String> it = assembledTrains.iterator();
                        JointArrDep joint = departure.getJointArrDep();
                        if (joint != null && it.hasNext()) {
                            // check departure order
                            String t = it.next();
                            boolean check = false;
                            Departure dep = assigned.get(t);
                            for (JointArrDepEntry entry :
                                departure.getJointArrDep().getEntryList()) {
                                if (dep == null) {
                                    logger.log(Level.INFO, "Departure order "
                                            + "error : " + t
                                            + " is not assigned to a departure.");
                                    valid = false;
                                    check = true;
                                    break;
                                }
                                if (entry.getArrDep().equals(dep.getName())) {
                                    if (it.hasNext()) {
                                        t = it.next();
                                        dep = assigned.get(t);
                                    } else {
                                        check = true;
                                        break;
                                    }
                                } else {
                                    Train other = covered.get(entry.getArrDep());
                                    if (other != null) {
                                        check = false;
                                        break;
                                    }
                                }
                            }
                            if (!check) {
                                logger.log(Level.INFO, "Departure order "
                                        + "error : " + joint.getName()
                                        + " order is not respected or is covered by"
                                        + " non assembled trains.");
                                valid = false;
                            }

                            // check minimum assembly time
                            if (lastAssemblyTime != null) {
                                if ((fill.getDate().getTime() - lastAssemblyTime.getTime())
                                        < parameters.getMinAsbTime().getTime()) {
                                    logger.log(Level.INFO, "Departure order "
                                            + "error : " + joint.getName()
                                            + " minimum assembly time is not respected.");
                                    valid = false;
                                    break;
                                }
                            }
                        }
                    }
                    break;
                }
                previous = fill;
            }
        }

        return valid;
    }

}
