// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Arrival;
import com.sncf.checker.bo.Departure;
import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.DepartureDAO;
import com.sncf.checker.io.TrainDAO;

/**
 * Required DBM for a departure.
 *
 * For any train and any departure, train remaining DBM should be greater than
 * the required DBM.
 *
 * @author YDS
 *
 */
public class DepartureRequiredDBMConstraint implements Constraint {

    /** Interface to Data layer. */
    private DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public DepartureRequiredDBMConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public final boolean validate() throws ROADEFException {
        HashMap<Departure, Train> assigned =
                new HashMap<Departure, Train>();
        TrainDAO dao = factory.getTrainDAO();
        DepartureDAO departureDAO = factory.getDepartureDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = dao.loadAll();
        for (Train train : trainList) {
            for (Fill fill : train.getFill()) {
                if (fill.getEvent().equals(Event.Departure)) {
                    Departure dep = departureDAO.loadByName(
                            fill.getComplement());
                    if (dep != null) {
                        assigned.put(dep, train);
                    }
                }
            }
        }

        for (Map.Entry<Departure, Train> entry : assigned.entrySet()) {
            Train train = entry.getValue();
            Departure departure = entry.getKey();
            int remDBM = train.getRemDBM();
            Arrival arrival = train.getArrival();
            if (arrival != null) {
                Departure linkedDep = arrival.getLinkedDep();
                if (linkedDep != null) {
                    Train prevTrain = assigned.get(linkedDep);
                    if (prevTrain != null) {
                        continue;
                    }
                }
            }

            for (Fill fill : train.getFill()) {
                if (fill.getEvent().equals(Event.BegMaintenance)) {
                    if (fill.getComplement().equals("D")) {
                        remDBM = train.getCategory().getMaxDBM();
                    }
                }
            }

            while (departure != null) {
                if (remDBM < departure.getReqDBM()) {
                    logger.log(Level.INFO, "Required DBM for a departure "
                            + "error : " + train.getName()
                            + " DBM is not sufficient for "
                            + departure.getName() + " ("
                            + departure.getReqDBM() + " required, "
                            + remDBM + " remaining).");
                    valid = false;
                    break;
                }
                remDBM = remDBM - departure.getReqDBM();
                if (departure.getLinkedArr() != null) {
                    train = departure.getLinkedArr().getTrain();
                    departure = null;
                    for (Fill fill : train.getFill()) {
                        if (fill.getEvent().equals(Event.BegMaintenance)) {
                            if (fill.getComplement().equals("D")) {
                                remDBM = train.getCategory().getMaxDBM();
                            }
                        } else if (fill.getEvent().equals(Event.Departure)) {
                            departure = departureDAO.loadByName(
                                    fill.getComplement());
                        }
                    }
                } else {
                    departure = null;
                }
            }
        }
        return valid;
    }
}
