// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Arrival;
import com.sncf.checker.bo.Departure;
import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.Duration;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.DepartureDAO;
import com.sncf.checker.io.TrainDAO;

/**
 * Required TBM for a departure.
 *
 * For any train and any departure, train remaining TBM should be greater than
 * the required TBM.
 *
 * @author YDS
 *
 */
public class DepartureRequiredTBMConstraint implements Constraint {

    /** Interface to Data layer. */
    private DAOFactory factory;

    /**
     * Constructor.
     *
     * @param factory dao factory to set
     */
    public DepartureRequiredTBMConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    /** {@inheritDoc} */
    public final boolean validate() throws ROADEFException {
        TrainDAO dao = factory.getTrainDAO();
        DepartureDAO departureDAO = factory.getDepartureDAO();
        boolean valid = true;
        HashMap<Departure, Train> assigned = new HashMap<Departure, Train>();
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = dao.loadAll();
        for (Train train : trainList) {
            for (Fill fill : train.getFill()) {
                if (fill.getEvent().equals(Event.Departure)) {
                    Departure dep = departureDAO.loadByName(fill.getComplement());
                    if (dep != null) {
                        assigned.put(dep, train);
                    }
                }
            }
        }

        for (Map.Entry<Departure, Train> entry : assigned.entrySet()) {
            Train train = entry.getValue();
            Departure departure = entry.getKey();
            Duration remTBM = train.getRemTBM();
            Arrival arrival = train.getArrival();
            if (arrival != null) {
                Departure linkedDep = arrival.getLinkedDep();
                if (linkedDep != null) {
                    Train prevTrain = assigned.get(linkedDep);
                    if (prevTrain != null) {
                        continue;
                    }
                }
            }

            for (Fill fill : train.getFill()) {
                if (fill.getEvent().equals(Event.BegMaintenance)) {
                    if (fill.getComplement().equals("T")) {
                        remTBM = train.getCategory().getMaxTBM();
                    }
                }
            }

            while (departure != null) {
                if (remTBM.getTime() < departure.getReqTBM().getTime()) {
                    logger.log(Level.INFO, "Required TBM for a departure error : "
                            + train.getName() + " TBM is not sufficient for "
                            + departure.getName() + " (" + departure.getReqTBM().toString() + " required, "
                            + remTBM.toString() + " remaining).");
                    valid = false;
                    break;
                }
                remTBM = remTBM.substract(departure.getReqTBM());
                if (departure.getLinkedArr() != null) {
                    train = departure.getLinkedArr().getTrain();
                    departure = null;
                    for (Fill fill : train.getFill()) {
                        if (fill.getEvent().equals(Event.BegMaintenance)) {
                            if (fill.getComplement().equals("T")) {
                                remTBM = train.getCategory().getMaxTBM();
                            }
                        } else if (fill.getEvent().equals(Event.Departure)) {
                            departure = departureDAO.loadByName(fill.getComplement());
                        }
                    }
                } else {
                    departure = null;
                }
            }
        }
        return valid;
    }
}