// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.TrackGroup;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.Duration;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.TrainDAO;

public class ResourceUseMinimumDurationConstraint implements Constraint {

    private DAOFactory factory;

    public ResourceUseMinimumDurationConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    public boolean validate() throws ROADEFException {
        TrainDAO dao = factory.getTrainDAO();
        Duration minResTime = factory.getParametersDAO().getParameters().getMinResTime();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        for (Train train : dao.loadAll()) {
            IState state = new DefaultState(minResTime);
            for (Fill fill : train.getFill()) {
                 state = state.handle(fill, logger);
            }
            if (!state.isValid()) {
                valid = false;
            }
        }

        return valid;
    }

    private interface IState {

        IState handle(final Fill fill, Logger logger);
        boolean isValid();
    }

    private class DefaultState implements IState {
        private boolean valid;
        Duration minResTime;

        public DefaultState(Duration minResTime) {
            valid = true;
            this.minResTime = minResTime;
        }

        public DefaultState(Duration minResTime, boolean valid) {
            this.valid = valid;
            this.minResTime = minResTime;
        }

        public IState handle(final Fill fill, Logger logger)
        {
            if (fill.getEvent().equals(Event.EnterResource)
                    && fill.getResource().getClass() != TrackGroup.class
                    && fill.getDate().getTime() != Datetime.MIN.getTime()) {
                return new ResourceState(minResTime, fill, valid);
            }
            return this;
        }

        public boolean isValid() {
            return valid;
        }
    }

    private class ResourceState implements IState {
        private Fill begin;
        private boolean valid;
        Duration minResTime;

        public ResourceState(Duration minResTime, final Fill fill, boolean valid) {
            begin = fill;
            this.valid = valid;
            this.minResTime = minResTime;
        }

        public IState handle(final Fill fill, Logger logger)
        {
            valid = true;
            if (fill.getEvent().equals(Event.ExitResource)) {
                int duration = fill.getDate().getTime() - begin.getDate().getTime();
                if (duration < minResTime.getTime() && fill.getDate().getTime() != Datetime.MAX.getTime()) {
                    logger.log(Level.INFO, "Minimum duration of use of a resource error : "
                            + fill.getTrain().getName() + " minimum resource time on "
                            + begin.getResource().getName() + " at " + begin.getDate().toString()
                            + " shoud be " + minResTime.toString() + " ("
                            + new Duration(duration).toString() + " found).");
                    valid = false;

                    return new DefaultState(minResTime, valid);
                }
            }
            return this;
        }

        public boolean isValid() {
            return valid;
        }
    }
}
