// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.sncf.checker.bo.Event;
import com.sncf.checker.bo.Fill;
import com.sncf.checker.bo.TrackGroup;
import com.sncf.checker.bo.Train;
import com.sncf.checker.common.Datetime;
import com.sncf.checker.common.Duration;
import com.sncf.checker.common.ROADEFException;
import com.sncf.checker.io.DAOFactory;
import com.sncf.checker.io.TrainDAO;

public class TrackGroupUseConstraint implements Constraint {

    private DAOFactory factory;

    public TrackGroupUseConstraint(final DAOFactory factory) {
        this.factory = factory;
    }

    public boolean validate() throws ROADEFException {
        TrainDAO trainDAO = factory.getTrainDAO();
        boolean valid = true;
        Logger logger = Logger.getLogger("logger");

        List<Train> trainList = trainDAO.loadAll();
        for (Train train : trainList) {
            IState state = new DefaultState();
            for (Fill fill : train.getFill()) {
                 state = state.handle(fill, logger);
            }
            if (!state.isValid()) {
                valid = false;
            }
        }

        return valid;
    }

    private interface IState {

        IState handle(final Fill fill, Logger logger);
        boolean isValid();
    }

    private class DefaultState implements IState {
        private boolean valid;

        public DefaultState() {
            valid = true;
        }

        public DefaultState(boolean valid) {
            this.valid = valid;
        }

        public IState handle(final Fill fill, Logger logger)
        {
            if (fill.getEvent().equals(Event.EnterResource) && fill.getResource().getClass() == TrackGroup.class) {
                return new TrackGroupState(fill, valid);
            }
            return this;
        }

        public boolean isValid() {
            return valid;
        }
    }

    private class TrackGroupState implements IState {
        private Fill begin;
        private boolean valid;

        public TrackGroupState(final Fill fill, boolean valid) {
            begin = fill;
            this.valid = valid;
        }

        public IState handle(final Fill fill, Logger logger)
        {
            if (fill.getEvent().equals(Event.ExitResource)) {
                TrackGroup trackGroup = (TrackGroup) begin.getResource();
                int duration = fill.getDate().getTime() - begin.getDate().getTime();
                if (duration != trackGroup.getTravelTime().getTime() && fill.getDate().getTime() != Datetime.MAX.getTime()) {
                    logger.log(Level.INFO, "Duration of use of track groups error : "
                            + fill.getTrain().getName() + " travel time on "
                            + begin.getResource().getName() + " at " + begin.getDate().toString()
                            + " shoud be " + trackGroup.getTravelTime().toString() + " ("
                            + new Duration(duration).toString() + " found).");
                    valid = false;
                }
                if (!(begin.getGate().isEmpty() || fill.getGate().isEmpty())) {
                    if (begin.getGate().charAt(0) == fill.getGate().charAt(0)) {
                        logger.log(Level.INFO, "Duration of use of track groups error : "
                                + fill.getTrain().getName() + " ExitResource event on "
                                + begin.getResource().getName() + " at " + fill.getDate().toString()
                                + " sould occur on the opposite side (entry gate " + begin.getGate()
                                + ", exit gate " + fill.getGate() + ").");
                        valid = false;
                    }
                }
            } else {
                logger.log(Level.INFO, "Duration of use of track groups error : "
                        + fill.getTrain().getName() + " EnterResource event on "
                        + begin.getResource().getName() + " at " + begin.getDate().toString()
                        + " must be followed by an ExitResource event");
                valid = false;
            }
            return new DefaultState(valid);
        }

        public boolean isValid() {
            return valid;
        }
    }
}
