// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.Facility;

/**
 * The data source is a tabulation separated file containing the maintenance facilitie.
 *
 * @author YDS
 *
 */
public class MaintenanceFacilityFlatFileDAO extends FlatFileDAO implements MaintenanceFacilityDAO {

    /** scenario file path. */
    private final String scenario;
    /** indexed by name maintenance facility map of the scenario. */
    private Map<String, Facility> cache = null;

    /**
     * Constructor.
     *
     * @param scenario file path to the scenario zip archive
     * @param solution file path to the solution file
     */
    public MaintenanceFacilityFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    /** TrainCategory DAO. */
    private TrainCategoryDAO trainCategoryDAO;

    /**
     * TrainCategory DAO setter.
     *
     * @param dao TrainCategory DAO
     */
    public void setTrainCategoryDAO(final TrainCategoryDAO dao) {
        this.trainCategoryDAO = dao;
    }

    /** Fill DAO. */
    private FillDAO fillDAO;

    /**
     * Fill DAO setter.
     *
     * @param dao Fill DAO
     */
    public void setFillDAO(final FillDAO dao) {
        this.fillDAO = dao;
    }

    /** Gate DAO. */
    private GateDAO gateDAO;

    /**
     * Gate DAO setter.
     *
     * @param dao Gate DAO
     */
    public void setGateDAO(final GateDAO dao) {
        this.gateDAO = dao;
    }

    /**
     * Load a facility by name.
     *
     * @param name the searched facility name
     * @return facility, null if not found
     */
    public Facility loadByName(final String name) {
        initialize();
        Facility entity = cache.get(name);
        return entity;
    }

    /**
     * Load all facilities.
     *
     * @return facility list
     */
    public List<Facility> loadAll() {
        initialize();
        ArrayList<Facility> list = new ArrayList<Facility>();
        list.addAll(cache.values());
        return list;
    }

    /**
     * Extract all facilities from facilities.csv in zip file.
     */
    private void initialize() {
        if (cache == null) {
            cache = new TreeMap<String, Facility>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "facilities.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";");
                    Facility temp;

                    temp = deserialize(splits);
                    cache.put(temp.getName(), temp);
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a MaintenanceFacility object.
     *
     * @param entry data extracted from the file
     * @return maintenance facility
     */
    private Facility deserialize(final String[] raw) {
        Facility entity = new Facility(gateDAO, fillDAO, trainCategoryDAO);

        entity.setName(raw[0]);
        entity.setType(raw[1]);
        entity.setLength(Integer.parseInt(raw[2]));

        return entity;
    }
}
