// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.SingleTrack;

/**
 * The data source is a tabulation separated file containing the single tracks.
 *
 * @author YDS
 *
 */
public class SingleTrackFlatFileDAO extends FlatFileDAO implements SingleTrackDAO {

    /** scenario file path. */
    private final String scenario;
    /** indexed by name single track map of the scenario. */
    private Map<String, SingleTrack> cache = null;

    /**
     * Constructor.
     *
     * @param scenario file path to the scenario zip archive
     * @param solution file path to the solution file
     */
    public SingleTrackFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    /** TrainCategory DAO. */
    private TrainCategoryDAO trainCategoryDAO;

    /**
     * TrainCategory DAO setter.
     *
     * @param dao TrainCategory DAO
     */
    public void setTrainCategoryDAO(final TrainCategoryDAO dao) {
        this.trainCategoryDAO = dao;
    }

    /** Fill DAO. */
    private FillDAO fillDAO;

    /**
     * Fill DAO setter.
     *
     * @param dao Fill DAO
     */
    public void setFillDAO(final FillDAO dao) {
        this.fillDAO = dao;
    }

    /** Gate DAO. */
    private GateDAO gateDAO;

    /**
     * Gate DAO setter.
     *
     * @param dao Gate DAO
     */
    public void setGateDAO(final GateDAO dao) {
        this.gateDAO = dao;
    }

    /**
     * Load a single track by name.
     *
     * @param name the searched single track name
     * @return single track object, null if not found
     */
    public SingleTrack loadByName(final String name) {
        initialize();
        SingleTrack entity = cache.get(name);
        return entity;
    }

    /**
     * Load all single tracks.
     *
     * @return single track list
     */
    public List<SingleTrack> loadAll() {
        initialize();
        ArrayList<SingleTrack> list = new ArrayList<SingleTrack>();
        list.addAll(cache.values());
        return list;
    }

    /**
     * Extract all single tracks from singleTracks.csv in zip file.
     */
    private void initialize() {
        if (cache == null) {
            cache = new TreeMap<String, SingleTrack>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "singleTracks.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";");
                    SingleTrack temp;

                    temp = deserialize(splits);
                    cache.put(temp.getName(), temp);
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a SingleTrack object.
     *
     * @param entry data extracted from the file
     * @return single track
     */
    private SingleTrack deserialize(final String[] raw) {
        SingleTrack entity = new SingleTrack(gateDAO, fillDAO, trainCategoryDAO);

        entity.setName(raw[0]);
        entity.setLength(Integer.parseInt(raw[1]));
        entity.setCapacity(Integer.parseInt(raw[2]));

        return entity;
    }
}
