// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.TrainCategory;
import com.sncf.checker.common.Duration;
import com.sncf.checker.common.TreeMultiMap;

/**
 * The data source is a tabulation separated file containing the train categories.
 *
 * @author YDS
 *
 */
public class TrainCategoryFlatFileDAO extends FlatFileDAO implements TrainCategoryDAO {

    /** scenario file path. */
    private final String scenario;
    /** indexed by name category map of the scenario. */
    private Map<String, TrainCategory> cache = null;
    /** ArrDepSequence indexed by name. */
    private TreeMultiMap<String, TrainCategory> departureIndexedCache = null;
    /** ArrDepSequence indexed by name. */
    private TreeMultiMap<String, TrainCategory> resourceIndexedCache = null;

    /**
     * Constructor.
     *
     * @param scenario file path to the scenario zip archive
     * @param solution file path to the solution file
     */
    public TrainCategoryFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    /**
     * Load a train category by name.
     *
     * @param name the searched category name
     * @return train category object, null if not found
     */
    public TrainCategory loadByName(final String name) {
        initialize();
        TrainCategory entity = cache.get(name);
        return entity;
    }

    /**
     * load the compatible category's list by departure name.
     *
     * @param name departure's name
     * @return train category, null if not found
     */
    public List<TrainCategory> loadByDeparture(final String name) {
        initialize();
        List<TrainCategory> list = departureIndexedCache.get(name);
        return list;
    }

    /**
     * load the compatible category's list by resource name.
     *
     * @param name resource's name
     * @return train category, null if not found
     */
    public List<TrainCategory> loadByResource(final String name) {
        initialize();
        List<TrainCategory> list = resourceIndexedCache.get(name);
        return list;
    }

    /**
     * Extract all categories from trainCategories.csv in zip file.
     */
    private void initialize() {
        if (cache == null || departureIndexedCache == null || resourceIndexedCache == null) {
            cache = new TreeMap<String, TrainCategory>();
            departureIndexedCache = new TreeMultiMap<String, TrainCategory>();
            resourceIndexedCache = new TreeMultiMap<String, TrainCategory>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "trainCategories.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";");
                    TrainCategory temp;

                    temp = deserialize(splits);
                    cache.put(temp.getName(), temp);
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }

            try {
                String current;

                String csvName = "compCatDep.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";");
                    departureIndexedCache.put(splits[0], loadByName(splits[1]));
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }

            try {
                String current;

                String csvName = "compCatRes.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";");
                    resourceIndexedCache.put(splits[0], loadByName(splits[1]));
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a TrainCategory object.
     *
     * @param entry data extracted from the file
     * @return train category
     */
    private TrainCategory deserialize(final String[] entry) {
        TrainCategory entity = new TrainCategory();

        entity.setName(entry[0]);
        entity.setLength(Integer.parseInt(entry[1]));
        entity.setCatGroup(entry[2]);
        entity.setMaxDBM(Integer.parseInt(entry[3]));
        entity.setMaxTBM(new Duration(entry[4]));
        entity.setMaintTimeD(new Duration(entry[5]));
        entity.setMaintTimeT(new Duration(entry[6]));

        return entity;
    }
}
