// Copyright 2013 SNCF. All rights reserved.
package com.sncf.checker.io;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.ZipInputStream;

import com.sncf.checker.bo.Yard;

/**
 * The data source is a tabulation separated file containing the yards.
 *
 * @author YDS
 *
 */
public class YardFlatFileDAO extends FlatFileDAO implements YardDAO {

    /** scenario file path. */
    private final String scenario;
    /** indexed by name yard map of the scenario. */
    private Map<String, Yard> cache = null;

    /**
     * Constructor.
     *
     * @param scenario file path to the scenario zip archive
     * @param solution file path to the solution file
     */
    public YardFlatFileDAO(final String scenario, final String solution) {
        this.scenario = scenario;
    }

    /** TrainCategory DAO. */
    private TrainCategoryDAO trainCategoryDAO;

    /**
     * TrainCategory DAO setter.
     *
     * @param dao TrainCategory DAO
     */
    public void setTrainCategoryDAO(final TrainCategoryDAO dao) {
        this.trainCategoryDAO = dao;
    }

    /** Fill DAO. */
    private FillDAO fillDAO;

    /**
     * Fill DAO setter.
     *
     * @param dao Fill DAO
     */
    public void setFillDAO(final FillDAO dao) {
        this.fillDAO = dao;
    }

    /** Gate DAO. */
    private GateDAO gateDAO;

    /**
     * Gate DAO setter.
     *
     * @param dao Gate DAO
     */
    public void setGateDAO(final GateDAO dao) {
        this.gateDAO = dao;
    }

    /**
     * Load a yard by name.
     *
     * @param name the searched yard name
     * @return yard object, null if not found
     */
    public Yard loadByName(final String name) {
        initialize();
        Yard entity = cache.get(name);
        return entity;
    }

    /**
     * Load all yards.
     *
     * @return yard list
     */
    public List<Yard> loadAll() {
        initialize();
        ArrayList<Yard> list = new ArrayList<Yard>();
        list.addAll(cache.values());
        return list;
    }

    /**
     * Extract all yards from yards.csv in zip file.
     */
    private void initialize() {
        if (cache == null) {
            cache = new TreeMap<String, Yard>();

            BufferedReader bufferedReader = null;
            ZipInputStream zis = null;

            try {
                String current;

                String csvName = "yards.csv";
                zis = getZipEntryInputStream(scenario, csvName);
                bufferedReader = new BufferedReader(new InputStreamReader(zis));

                bufferedReader.readLine();
                while ((current = bufferedReader.readLine()) != null) {
                    String[] splits = current.split(";");
                    Yard temp;

                    temp = deserialize(splits);
                    cache.put(temp.getName(), temp);
                }
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                if (bufferedReader != null) {
                    try {
                        bufferedReader.close();
                        zis.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }

    /**
     * Deserialize an entry in a Yard object.
     *
     * @param entry data extracted from the file
     * @return yard
     */
    private Yard deserialize(final String[] raw) {
        Yard entity = new Yard(gateDAO, fillDAO, trainCategoryDAO);

        entity.setName(raw[0]);
        entity.setCapacity(Integer.parseInt(raw[1]));

        return entity;
    }
}
